#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAX_LINE_LENGTH 100

typedef struct Node {
    char text[MAX_LINE_LENGTH];
    struct Node *yes;
    struct Node *no;
} Node;

Node* create_node(const char* text) {
    Node *node = (Node*)malloc(sizeof(Node));
    if (!node) {
        fprintf(stderr, "Memory allocation error.\n");
        exit(1);
    }
    strncpy(node->text, text, MAX_LINE_LENGTH);
    node->yes = NULL;
    node->no = NULL;
    return node;
}

Node* parse_knowledge_base(FILE *file) {
    char line[MAX_LINE_LENGTH];

    // Считываем строку из файла
    if (!fgets(line, sizeof(line), file) || line[0] == '\n') {
        return NULL;
    }

    // Удаляем символ новой строки
    line[strcspn(line, "\n")] = 0;

    // Создаем узел, проверяем, является ли это ответом (начинается с '*')
    Node *node = NULL;
    if (line[0] == '*') {
        node = create_node(line + 1);  // Создаем узел-ответ
    } else {
        node = create_node(line);      // Создаем узел-вопрос
        node->yes = parse_knowledge_base(file);  // Переход для ответа "да"
        node->no = parse_knowledge_base(file);   // Переход для ответа "нет"
    }
    return node;
}


int count_products(Node *node) {
    if (!node) return 0;
    if (!node->yes && !node->no) return 1;
    return count_products(node->yes) + count_products(node->no);
}

void run_expert_system(Node *node, FILE *input) {
    while (node) {
        if (!node->yes && !node->no) {
            printf("*%s\n", node->text);
            printf("Koniec\n");
            return;
        }

        printf("Odpovedajte 'a' pre prvu moznost alebo 'n' pre druhu moznost.\n");
        printf("%s\n", node->text);

        char answer;
        if (fscanf(input, " %c", &answer) != 1) {
            printf("Koniec\n");
            return;
        }

        if (answer == 'a') {
            node = node->yes;
        } else if (answer == 'n') {
            node = node->no;
        } else {
            printf("Nespravny vstup.\nKoniec\n");
            return;
        }
    }
}


void free_tree(Node *node) {
    if (node) {
        free_tree(node->yes);
        free_tree(node->no);
        free(node);
    }
}

void create_default_knowledge_base() {
    FILE *file = fopen("knowledge_base.txt", "w");
    if (!file) {
        perror("Failed to create the knowledge base file");
        exit(1);
    }
    fprintf(file, "Je to ovocie alebo zelenina\n");
    fprintf(file, "*Jablko\n");
    fprintf(file, "*Mrkva\n");
    fprintf(file, "\n");
    fprintf(file, "a\n");
    fclose(file);
}

int main() {
    FILE *file = fopen("knowledge_base.txt", "r");
    if (!file) {
        create_default_knowledge_base();
        file = fopen("knowledge_base.txt", "r");
        if (!file) {
            perror("Failed to open the knowledge base file");
            return 1;
        }
    }

    Node *root = parse_knowledge_base(file);
    if (!root) {
        printf("Báza znalostí sa nedá načítať.\n");
        fclose(file);
        return 1;
    }

    printf("Expert z bufetu to vie.\n");
    int product_count = count_products(root);
    printf("Pozna %d druhov ovocia a zeleniny.\n", product_count);

    char line[MAX_LINE_LENGTH];
    while (fgets(line, sizeof(line), file) && line[0] != '\n');

    run_expert_system(root, file);

    free_tree(root);
    fclose(file);
    return 0;
}


